@@ -34,6 +34,7 @@
  * SPI parts have 16-bit device IDs if they support RDID.
  */
 
+#define GENERIC_MANUF_ID	0xffff	/* Check if there is a vendor ID */
 #define GENERIC_DEVICE_ID	0xffff	/* Only match the vendor ID */
 
 #define ALLIANCE_ID		0x52	/* Alliance Semiconductor */
@@ -163,13 +164,29 @@
 #define EON_ID			0x7F1C	/* EON Silicon Devices */
 #define EON_ID_NOPREFIX		0x1C	/* EON, missing 0x7F prefix */
 #define EN_25B05		0x2010	/* Same as P05, 2^19 kbit or 2^16 kByte */
+#define EN_25B05T		0x25
+#define EN_25B05B		0x95
 #define EN_25B10		0x2011	/* Same as P10 */
+#define EN_25B10T		0x40
+#define EN_25B10B		0x30
 #define EN_25B20		0x2012	/* Same as P20 */
+#define EN_25B20T		0x41
+#define EN_25B20B		0x31
 #define EN_25B40		0x2013	/* Same as P40 */
+#define EN_25B40T		0x42
+#define EN_25B40B		0x32
 #define EN_25B80		0x2014	/* Same as P80 */
+#define EN_25B80T		0x43
+#define EN_25B80B		0x33
 #define EN_25B16		0x2015	/* Same as P16 */
+#define EN_25B16T		0x44
+#define EN_25B16B		0x34
 #define EN_25B32		0x2016	/* Same as P32 */
+#define EN_25B32T		0x45
+#define EN_25B32B		0x35
 #define EN_25B64		0x2017	/* Same as P64 */
+#define EN_25B64T		0x46
+#define EN_25B64B		0x36
 #define EN_25D16		0x3015
 #define EN_25F05		0x3110
 #define EN_25F10		0x3111
@@ -179,7 +196,7 @@
 #define EN_25F16		0x3115
 #define EN_25F32		0x3116
 #define EN_29F512		0x7F21
-#define EN_29F010		0x7F20
+#define EN_29F010		0x20
 #define EN_29F040A		0x7F04
 #define EN_29LV010		0x7F6E
 #define EN_29LV040A		0x7F4F	/* EN_29LV040(A) */
@@ -237,8 +254,15 @@
 #define INTEL_ID		0x89	/* Intel */
 #define I_82802AB		0xAD
 #define I_82802AC		0xAC
+#define E_28F004S5		0xA7
+#define E_28F008S5		0xA6
+#define E_28F016S5		0xAA
 #define P28F001BXT		0x94	/* 28F001BX-T */
 #define P28F001BXB		0x95	/* 28F001BX-B */
+#define P28F004BT		0x78	/* 28F004BV/BE-T */
+#define P28F004BB		0x79	/* 28F004BV/BE-B */
+#define P28F400BT		0x70	/* 28F400BV/CV/CE-T */
+#define P28F400BB		0x71	/* 28F400BV/CV/CE-B */
 #define SHARP_LH28F008SA	0xA2	/* Sharp chip, Intel Vendor ID */
 #define SHARP_LH28F008SC	0xA6	/* Sharp chip, Intel Vendor ID */
 
@@ -251,11 +275,11 @@
  * and use the same set of IDs.
  */
 #define MX_ID			0xC2	/* Macronix (MX) */
-#define MX_25L512		0x2010	/* 2^19 kbit or 2^16 kByte */
+#define MX_25L512		0x2010	/* Same as MX25V512 */
 #define MX_25L1005		0x2011
 #define MX_25L2005		0x2012
 #define MX_25L4005		0x2013	/* MX25L4005{,A} */
-#define MX_25L8005		0x2014
+#define MX_25L8005		0x2014	/* Same as MX25V8005 */
 #define MX_25L1605		0x2015	/* MX25L1605{,A,D} */
 #define MX_25L3205		0x2016	/* MX25L3205{,A} */
 #define MX_25L6405		0x2017	/* MX25L3205{,D} */
@@ -325,6 +349,14 @@
 #define PMC_49FL002		0x6D
 #define PMC_49FL004		0x6E
 
+/* 
+ * The Sanyo chip found so far uses SPI, first byte is manufacture code,
+ * second byte is the device code,
+ * third byte is a dummy byte.
+ */
+#define SANYO_ID		0x62
+#define SANYO_LE25FW203A	0x1600
+
 #define SHARP_ID		0xB0	/* Sharp */
 #define SHARP_LH28F008BJxxPT	0xEC
 #define SHARP_LH28F008BJxxPB	0xED
@@ -340,6 +372,7 @@
  * the second device ID byte is memory capacity.
  */
 #define SPANSION_ID		0x01	/* Spansion, same ID as AMD */
+#define SPANSION_S25FL008A	0x0213
 #define SPANSION_S25FL016A	0x0214
 
 /*
@@ -380,6 +413,7 @@
 #define SST_29VF020		0x25
 #define SST_29SF040		0x13
 #define SST_29VF040		0x14
+#define SST_39SF512		0xB4
 #define SST_39SF010		0xB5
 #define SST_39SF020		0xB6	/* Same as 39SF020A */
 #define SST_39SF040		0xB7
@@ -446,6 +480,7 @@
 #define ST_M29F800DT		0xEC
 #define ST_M29W010B		0x23
 #define ST_M29W040B		0xE3
+#define ST_M29W512B		0x27
 
 #define SYNCMOS_ID		0x40	/* SyncMOS and Mosel Vitelic */
 #define S29C51001T		0x01
@@ -471,6 +506,9 @@
 #define W_25X16			0x3015
 #define W_25X32			0x3016
 #define W_25X64			0x3017
+#define W_25Q80			0x4014
+#define W_25Q16			0x4015
+#define W_25Q32			0x4016
 #define W_29C011		0xC1
 #define W_29C020C		0x45	/* Same as W29C020 and ASD AE29F2008 */
 #define W_29C040P		0x46	/* Same as W29C040 */
