@@ -27,6 +27,7 @@
 /* Read Electronic ID */
 #define JEDEC_RDID		0x9f
 #define JEDEC_RDID_OUTSIZE	0x01
+/* INSIZE may be 0x04 for some chips*/
 #define JEDEC_RDID_INSIZE	0x03
 
 /* AT25F512A has bit 3 as don't care bit in commands */
@@ -42,6 +43,7 @@
 /* Read Electronic Signature */
 #define JEDEC_RES		0xab
 #define JEDEC_RES_OUTSIZE	0x04
+/* INSIZE may be 0x02 for some chips*/
 #define JEDEC_RES_INSIZE	0x01
 
 /* Write Enable */
@@ -74,6 +76,11 @@
 #define JEDEC_BE_D8_OUTSIZE	0x04
 #define JEDEC_BE_D8_INSIZE	0x00
 
+/* Block Erase 0xd7 is supported by PMC chips. */
+#define JEDEC_BE_D7		0xd7
+#define JEDEC_BE_D7_OUTSIZE	0x04
+#define JEDEC_BE_D7_INSIZE	0x00
+
 /* Sector Erase 0x20 is supported by Macronix/SST chips. */
 #define JEDEC_SE		0x20
 #define JEDEC_SE_OUTSIZE	0x04
@@ -106,6 +113,7 @@
 #define JEDEC_BYTE_PROGRAM_INSIZE	0x00
 
 /* Error codes */
+#define SPI_GENERIC_ERROR	-1
 #define SPI_INVALID_OPCODE	-2
 #define SPI_INVALID_ADDRESS	-3
 #define SPI_INVALID_LENGTH	-4
