@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2000 Silicon Integrated System Corporation
  * Copyright (C) 2005 coresystems GmbH <stepan@openbios.org>
+ * Copyright (C) 2009 Sean Nelson <audiohacked@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
  */
 
 #include "flash.h"
+#include "chipdrivers.h"
 
 #define AUTO_PG_ERASE1		0x20
 #define AUTO_PG_ERASE2		0xD0
@@ -30,31 +32,27 @@
 
 static void protect_28sf040(chipaddr bios)
 {
-	uint8_t tmp;
-
-	tmp = chip_readb(bios + 0x1823);
-	tmp = chip_readb(bios + 0x1820);
-	tmp = chip_readb(bios + 0x1822);
-	tmp = chip_readb(bios + 0x0418);
-	tmp = chip_readb(bios + 0x041B);
-	tmp = chip_readb(bios + 0x0419);
-	tmp = chip_readb(bios + 0x040A);
+	chip_readb(bios + 0x1823);
+	chip_readb(bios + 0x1820);
+	chip_readb(bios + 0x1822);
+	chip_readb(bios + 0x0418);
+	chip_readb(bios + 0x041B);
+	chip_readb(bios + 0x0419);
+	chip_readb(bios + 0x040A);
 }
 
 static void unprotect_28sf040(chipaddr bios)
 {
-	uint8_t tmp;
-
-	tmp = chip_readb(bios + 0x1823);
-	tmp = chip_readb(bios + 0x1820);
-	tmp = chip_readb(bios + 0x1822);
-	tmp = chip_readb(bios + 0x0418);
-	tmp = chip_readb(bios + 0x041B);
-	tmp = chip_readb(bios + 0x0419);
-	tmp = chip_readb(bios + 0x041A);
+	chip_readb(bios + 0x1823);
+	chip_readb(bios + 0x1820);
+	chip_readb(bios + 0x1822);
+	chip_readb(bios + 0x0418);
+	chip_readb(bios + 0x041B);
+	chip_readb(bios + 0x0419);
+	chip_readb(bios + 0x041A);
 }
 
-static int erase_sector_28sf040(struct flashchip *flash, unsigned long address, int sector_size)
+int erase_sector_28sf040(struct flashchip *flash, unsigned int address, unsigned int sector_size)
 {
 	chipaddr bios = flash->virtual_memory;
 
@@ -65,13 +63,13 @@ static int erase_sector_28sf040(struct flashchip *flash, unsigned long address,
 	toggle_ready_jedec(bios);
 
 	if (check_erased_range(flash, address, sector_size)) {
-		fprintf(stderr, "ERASE FAILED!\n");
+		msg_cerr("ERASE FAILED!\n");
 		return -1;
 	}
 	return 0;
 }
 
-static int write_sector_28sf040(chipaddr bios, uint8_t *src, chipaddr dst,
+int write_sector_28sf040(chipaddr bios, uint8_t *src, chipaddr dst,
 				unsigned int page_size)
 {
 	int i;
@@ -94,30 +92,6 @@ static int write_sector_28sf040(chipaddr bios, uint8_t *src, chipaddr dst,
 	return 0;
 }
 
-int probe_28sf040(struct flashchip *flash)
-{
-	chipaddr bios = flash->virtual_memory;
-	uint8_t id1, id2;
-
-	chip_writeb(RESET, bios);
-	programmer_delay(10);
-
-	chip_writeb(READ_ID, bios);
-	programmer_delay(10);
-	id1 = chip_readb(bios);
-	programmer_delay(10);
-	id2 = chip_readb(bios + 0x01);
-
-	chip_writeb(RESET, bios);
-	programmer_delay(10);
-
-	printf_debug("%s: id1 0x%02x, id2 0x%02x\n", __FUNCTION__, id1, id2);
-	if (id1 == flash->manufacture_id && id2 == flash->model_id)
-		return 1;
-
-	return 0;
-}
-
 int erase_28sf040(struct flashchip *flash)
 {
 	chipaddr bios = flash->virtual_memory;
@@ -131,7 +105,7 @@ int erase_28sf040(struct flashchip *flash)
 	toggle_ready_jedec(bios);
 
 	if (check_erased_range(flash, 0, flash->total_size * 1024)) {
-		fprintf(stderr, "ERASE FAILED!\n");
+		msg_cerr("ERASE FAILED!\n");
 		return -1;
 	}
 	return 0;
@@ -146,23 +120,33 @@ int write_28sf040(struct flashchip *flash, uint8_t *buf)
 
 	unprotect_28sf040(bios);
 
-	printf("Programming page: ");
+	msg_cinfo("Programming page: ");
 	for (i = 0; i < total_size / page_size; i++) {
 		/* erase the page before programming */
 		if (erase_sector_28sf040(flash, i * page_size, page_size)) {
-			fprintf(stderr, "ERASE FAILED!\n");
+			msg_cerr("ERASE FAILED!\n");
 			return -1;
 		}
 
 		/* write to the sector */
-		printf("%04d at address: 0x%08x", i, i * page_size);
+		msg_cinfo("%04d at address: 0x%08x", i, i * page_size);
 		write_sector_28sf040(bios, buf + i * page_size,
 				     bios + i * page_size, page_size);
-		printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
+		msg_cinfo("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
 	}
-	printf("\n");
+	msg_cinfo("\n");
 
 	protect_28sf040(bios);
 
 	return 0;
 }
+
+int erase_chip_28sf040(struct flashchip *flash, unsigned int addr, unsigned int blocklen)
+{
+	if ((addr != 0) || (blocklen != flash->total_size * 1024)) {
+		msg_cerr("%s called with incorrect arguments\n",
+			__func__);
+		return -1;
+	}
+	return erase_28sf040(flash);
+}
